#include "userdata.h"
#include "userfile.h"
#include "extensions.h"
#include <curses.h>
#include <stdio.h>
/*
#include <stdlib.h>
#include <string.h>
#include <time.h>
#include <netinet/in.h> 
#include <sys/time.h>
#include <unistd.h>
#include <sys/types.h>
#include <netdb.h>
#include <ctype.h>
*/

#define STYLE_NORMAL 	1
#define STYLE_GREEN	2
#define STYLE_STDOUT	3
#define	STYLE_RED	4

extern int update_user(struct UF user);
extern struct USERDATA *first;
extern struct USERDATA *last;
extern char version[];

void display_question(char *help, char *reply);
void display_userdata(void);
void clear_window(void);
void enable_color(void);
void mvparse(int pars);
int vpos=0, hpos=1, numlines=3, max=0, win=-1, sections=-1;
int currentuser, max_y, max_x;

static WINDOW *infowin;
static WINDOW *mainmenu;

mainMenu() {
 int ch, cnt;
 char inbuf[255];
 struct aclmember *entry = NULL;

 initscr();
 cbreak();
 nonl();
 intrflush(stdscr, FALSE);
 keypad(stdscr, TRUE);
 clear();
 
 start_color();
 init_pair(STYLE_NORMAL, COLOR_CYAN, COLOR_BLACK);
 init_pair(STYLE_GREEN, COLOR_BLACK, COLOR_CYAN);
 init_pair(STYLE_STDOUT, COLOR_MAGENTA, COLOR_BLACK);
 init_pair(STYLE_RED, COLOR_RED, COLOR_BLACK);

 mainmenu=newwin(0, 0, 0, 0);
 getmaxyx(mainmenu, max_y, max_x);	
 delwin(mainmenu);

 mainmenu=newwin(max_y, max_x, 0, 0);
 wattron(mainmenu, COLOR_PAIR(STYLE_NORMAL));
 wborder(mainmenu, 0,0,0,0,0,0,0,0);
 wattroff(mainmenu, COLOR_PAIR(STYLE_NORMAL));
 wattron(mainmenu, COLOR_PAIR(STYLE_NORMAL));
 mvwprintw(mainmenu, 1, 4, "%s", version);
 wattroff(mainmenu, COLOR_PAIR(STYLE_NORMAL));
 wattron(mainmenu, A_BOLD);
 mvwhline(mainmenu, 2, 1, 0, max_x - 2);
 mvwvline(mainmenu, 3, 13, 0, max_y - 4);
 wattroff(mainmenu, A_BOLD);
 leaveok(mainmenu, TRUE);   
 mvparse(max);

 while(getaclentry("stat_section", &entry)) sections++;
 while(1) {
   untouchwin(stdscr);
   memset(inbuf, '\0', sizeof(inbuf));
   curs_set(0);
   noecho();
   ch=getch();
   echo();    
   switch(ch) {	
     case KEY_UP:
      vpos--;
      if (vpos < 0) {
          vpos = 0;
	  max--;
      }
      mvparse(max);
      break;

     case KEY_DOWN:
      vpos++;
      if (vpos > (max_y - 5)) {
      	  vpos = (max_y - 5);
	  max++;
      }
      mvparse(max);	
      break;
     
     case KEY_NPAGE:
      max += (max_y - 4);
      mvparse(max);
      break;

     case KEY_PPAGE:
      max -= (max_y - 4);
      mvparse(max);
      break;

     case KEY_HOME:
      max = 0;
      vpos = 0;
      mvparse(max);
      break;

     case KEY_END:
      max = 32767;
      mvparse(max);
      break;
       
     case KEY_RIGHT: case 9:
      win++;
      if (win > sections) win = -1;
      display_userdata();
      break;

     case KEY_LEFT:
      win--;
      if (win < -1) win = sections;
      display_userdata();
      break;

     case 'q': case 27:
      clear();
      refresh();
      def_prog_mode();
      endwin();
      return;
      break;
     
     case 'm':
     case 'M':
      if (win != -1) break;
      display_question("Max Logins:", inbuf);
      parse(currentuser);
      ug.num_logins = atoi(inbuf);	
      updateuser(ug);
      display_userdata();
      break;

     case 'i':
     case 'I':
      if (win != -1) break;
      display_question("User Tagline:", inbuf);
      parse(currentuser);
      strncpy(ug.tagline, inbuf, sizeof(ug.tagline));
      updateuser(ug);
      display_userdata();
      break;

     case 'r':
     case 'R':
      display_question("User Ratio:", inbuf);
      parse(currentuser);
      if (win == -1) ug.statsection[0].ratio = atol(inbuf);
      else {
   	cnt = 0;
	entry = NULL;
	while (getaclentry("stat_section", &entry))
		if (win == cnt++) break;
	ug.statsection[win].ratio = atoi(inbuf);
      }
      updateuser(ug);
      display_userdata();
      break;
     
     case 'c':
     case 'C':
      display_question("User Credits (KB):", inbuf);
      parse(currentuser);
      if (win == -1) ug.statsection[0].credits = atol(inbuf);
      else {
        cnt = 0;
	entry = NULL;
	while (getaclentry("stat_section", &entry)) 
		if (win == cnt++) break;
	if (!strcasecmp(ARG2, "yes")) ug.statsection[win].credits = atol(inbuf);
        else ug.statsection[0].credits = atol(inbuf);
      }	
      updateuser(ug);
      display_userdata();
      break;

     case 'b':
     case 'B':
      if (win == -1) break;
      display_question("Bytes Nuked  (Bytes):", inbuf);
      parse(currentuser);
      ug.statsection[win].bytes_nuked = atol(inbuf);
      updateuser(ug);
      display_userdata();
      break;

     case 'n':
     case 'N':
      if (win == -1) break;
      display_question("Times Nuked:", inbuf);
      parse(currentuser);
      ug.statsection[win].times_nuked = atoi(inbuf);
      updateuser(ug);
      display_userdata();
      break;

     case 'a':
     case 'A':
      if (win != -1) break;
      display_question("User's Weekly Allotment (KB):", inbuf);
      parse(currentuser);
      ug.weekly_allotment = atol(inbuf);
      updateuser(ug);
      display_userdata();
      break;

     case 's':
     case 'S':
      if (win != -1) break;
      display_question("Max Download Speed (Bytes):", inbuf);
      parse(currentuser);
      ug.traffic_down = atol(inbuf);
      updateuser(ug);
      display_userdata();
      break;

     case 'P':
     case 'p':
      if (win != -1) break;
      display_question("Max Upload Speed (Bytes):", inbuf);
      parse(currentuser);
      ug.traffic_up = atol(inbuf);
      updateuser(ug);
      display_userdata();
      break;

     case 'f':
     case 'F':
      if (win != -1) break;
      display_question("User Flags:", inbuf);
      parse(currentuser);
      strncpy(ug.flags, inbuf, sizeof(ug.flags));
      updateuser(ug);
      display_userdata();
      break;

     case 'g':
     case 'G':
      if (win != -1) break;
      display_question("User GADMIN Slots:", inbuf);
      parse(currentuser);
      ug.gadmin_max = atoi(inbuf);
      updateuser(ug);
      display_userdata();
      break;

     case 'L':
     case 'l':
      if (win != -1) break;
      display_question("User Leech Slots:", inbuf);
      parse(currentuser);
      ug.gadmin_leech = atoi(inbuf);
      updateuser(ug);
      display_userdata();
      break;
      
     case 't':
     case 'T':
      if (win != -1) break;
      display_question("User's Time Limit (Minutes):", inbuf);
      parse(currentuser);
      ug.time_limit = atoi(inbuf);
      updateuser(ug);
      display_userdata();
      break;
      
     case '0':
      if (win != -1) break;
      display_question("IDENT@IP:", inbuf);
      parse(currentuser);
      strncpy(ug.ip[0], inbuf, sizeof(ug.ip[0]));
      updateuser(ug);
      display_userdata();
      break;
     case '1':
      if (win != -1) break;
      display_question("IDENT@IP:", inbuf);
      parse(currentuser);
      strncpy(ug.ip[1], inbuf, sizeof(ug.ip[1]));
      updateuser(ug);
      display_userdata();
      break;
     case '2':
      if (win != -1) break;
      display_question("IDENT@IP:", inbuf);
      parse(currentuser);
      strncpy(ug.ip[2], inbuf, sizeof(ug.ip[2]));
      updateuser(ug);
      display_userdata();
      break;
     case '3':
      if (win != -1) break;
      display_question("IDENT@IP:", inbuf);
      parse(currentuser);
      strncpy(ug.ip[3], inbuf, sizeof(ug.ip[3]));
      updateuser(ug);
      display_userdata();
      break;
     case '4':
      if (win != -1) break;
      display_question("IDENT@IP:", inbuf);
      parse(currentuser);
      strncpy(ug.ip[4], inbuf, sizeof(ug.ip[4]));
      updateuser(ug);
      display_userdata();
      break;
     case '5':
      if (win != -1) break;
      display_question("IDENT@IP:", inbuf);
      parse(currentuser);
      strncpy(ug.ip[5], inbuf, sizeof(ug.ip[5]));
      updateuser(ug);
      display_userdata();
      break;
     case '6':
      if (win != -1) break;
      display_question("IDENT@IP:", inbuf);
      parse(currentuser);
      strncpy(ug.ip[6], inbuf, sizeof(ug.ip[6]));
      updateuser(ug);
      display_userdata();
      break;
     case '7':
      if (win != -1) break;
      display_question("IDENT@IP:", inbuf);
      parse(currentuser);
      strncpy(ug.ip[7], inbuf, sizeof(ug.ip[7]));
      updateuser(ug);
      display_userdata();
      break;
     case '8':
      if (win != -1) break;
      display_question("IDENT@IP:", inbuf);
      parse(currentuser);
      strncpy(ug.ip[8], inbuf, sizeof(ug.ip[8]));
      updateuser(ug);
      display_userdata();
      break;
     case '9':
      if (win != -1) break;
      display_question("IDENT@IP:", inbuf);
      parse(currentuser);
      strncpy(ug.ip[9], inbuf, sizeof(ug.ip[9]));
      updateuser(ug);
      display_userdata();
      break;     	

     default:
      break;
    }
  }
}

void mvparse(int pos) {
	struct USERDATA *users = first;
	struct USERDATA *end = last;
	int col;

	if (max < 0) {	/* Top scroll stop. */
		max = 0;
		vpos = 0;
	}
	if (end->num < (max_y - 5)) {  /* Bottom Scroll Stop. */
		col = end->num;
		if (vpos > col) 
			vpos = col;
		if (max > col) {
			vpos = col;
			max = 0;
		}
	} else {
		col = end->num - (max_y - 5);
		if (max > col) {  
			vpos = (max_y - 5);
			max = col;
		}
	}
	col = 0;
	while (col < max) {
		users = users->next;
		col++;
	}
	col = 0;
	while (users && col < (max_y - 4)) {
		if (users->num == vpos+max) {
			wattron(mainmenu,A_REVERSE);
			mvwprintw(mainmenu, numlines+col, 1, "%-12.12s", users->username);
			wattroff(mainmenu,A_REVERSE);
			currentuser = users->num;
			parse(currentuser);
			display_userdata();
		} else {
			mvwprintw(mainmenu, numlines+col, 1, "%-12.12s", users->username);
		}
		col++;
		users = users->next;
	}
wrefresh(mainmenu);
}

/*------------------------------------------------------------------------*/
void display_question(char *help, char *reply) {
 	infowin=subwin(mainmenu, 6, 40, 10, 20);
	touchwin(stdscr);
	curs_set(1);
	wbkgdset(infowin, ' ' | COLOR_PAIR(STYLE_GREEN));
	werase(infowin);
	wbkgdset(infowin, ' ');
	wattrset(infowin, COLOR_PAIR(STYLE_GREEN));
 	box(infowin, 0, 0);
	wattron(infowin, COLOR_PAIR(STYLE_GREEN));
	mvwaddstr(infowin, 2, 4, help);
	wattron(infowin, A_REVERSE);
	mvwaddstr(infowin, 3, 4, "                                ");
	wattrset(infowin, A_NORMAL);
	wnoutrefresh(infowin);
	doupdate();
 	mvwgetnstr(infowin, 3, 4, reply, 31);
	werase(infowin);
	delwin(infowin);
	return;
}
/*------------------------------------------------------------------------*/
void display_userdata(void) {
    int cnt;
    char numbuff[64];
    struct aclmember *entry = NULL;

    clear_window();
    if (win == -1) {
	mvwprintw(mainmenu, 3, 15, "User:  %-24.24s", ug.name);
	mvwprintw(mainmenu, 4, 15, "Tagline:  %-50.50s", ug.tagline);
 	mvwprintw(mainmenu, 5, 15, "Flags: %-49.49s", ug.flags);
	mvwprintw(mainmenu, 6, 15, "Ratio: %d", ug.statsection[0].ratio);
	commafmt(numbuff, 64, ug.statsection[0].credits);
	mvwprintw(mainmenu, 7, 15, "Credits:  %-s KB", numbuff);
	commafmt(numbuff, 64, ug.weekly_allotment);
	mvwprintw(mainmenu, 8, 15, "Weekly Allotment:  %-s KB", numbuff);
	mvwprintw(mainmenu, 9, 15, "Max DL Speed:  %luB/s", ug.traffic_down);
	mvwprintw(mainmenu, 10, 15, "MAx UP Speed: %luB/s", ug.traffic_up);
	mvwprintw(mainmenu, 11, 15, "Max Simult Logins:  %d", ug.num_logins);
	mvwprintw(mainmenu,12, 15, "Primary Group:  %s", ug.group_user[0]);
	mvwprintw(mainmenu,13, 15, "Group Slots Avail:  %d", ug.gadmin_max);	
	mvwprintw(mainmenu,14, 15, "Group Leech Slots:  %d", ug.gadmin_leech);
	mvwprintw(mainmenu,15, 15, "Time Limit:  %d min.", ug.time_limit);
	mvwprintw(mainmenu,16, 15, "Logins:  %d", ug.login_times);
	mvwprintw(mainmenu,17, 15, "Last On:  %-45.45s", trim(ctime(&ug.last_on)));

	/* IDENT INFO */
	mvwhline(mainmenu, max_y - 7, 14, 0, 65);
	mvwaddstr(mainmenu, max_y - 7, 38, "[ Ident@IP: ]");
	mvwprintw(mainmenu, max_y - 6, 15, "0. %-30.30s", ug.ip[0]);
	mvwprintw(mainmenu, max_y - 5, 15, "1. %-30.30s", ug.ip[1]);
	mvwprintw(mainmenu, max_y - 4, 15, "2. %-30.30s", ug.ip[2]);
	mvwprintw(mainmenu, max_y - 3, 15, "3. %-30.30s", ug.ip[3]);
	mvwprintw(mainmenu, max_y - 2, 15, "4. %-30.30s", ug.ip[4]);
	mvwprintw(mainmenu, max_y - 6, 47, "5. %-29.29s", ug.ip[5]);
	mvwprintw(mainmenu, max_y - 5, 47, "6. %-29.29s", ug.ip[6]);
	mvwprintw(mainmenu, max_y - 4, 47, "7. %-29.29s", ug.ip[7]);
	mvwprintw(mainmenu, max_y - 3, 47, "8. %-29.29s", ug.ip[8]);
	mvwprintw(mainmenu, max_y - 2, 47, "9. %-29.29s", ug.ip[9]);

	wattron(mainmenu, COLOR_PAIR(STYLE_NORMAL));
	wattron(mainmenu, A_BOLD);
	mvwaddch(mainmenu, 4, 19, 'i'); /*Tagline */
	mvwaddch(mainmenu, 5, 15, 'F'); /*Flags */
	mvwaddch(mainmenu, 6, 15, 'R'); /*Ratio */
	mvwaddch(mainmenu, 7, 15, 'C'); /*Credits */
	mvwaddch(mainmenu, 8, 22, 'A'); /*Allotment */
	mvwaddch(mainmenu, 9, 22, 'S'); /*Dl Speed */
	mvwaddch(mainmenu, 10, 20, 'P'); /*UP Speed */
	mvwaddch(mainmenu, 11, 15, 'M'); /*Max Logins */
	mvwaddch(mainmenu, 13, 15, 'G'); /*Slots */
	mvwaddch(mainmenu, 14, 21, 'L'); /*Leech slots */
	mvwaddch(mainmenu, 15, 15, 'T'); /*Time limit */
	mvwaddch(mainmenu, max_y - 6, 15, '0');
	mvwaddch(mainmenu, max_y - 5, 15, '1');
	mvwaddch(mainmenu, max_y - 4, 15, '2');
	mvwaddch(mainmenu, max_y - 3, 15, '3');
	mvwaddch(mainmenu, max_y - 2, 15, '4');
	mvwaddch(mainmenu, max_y - 6, 47, '5');
	mvwaddch(mainmenu, max_y - 5, 47, '6');
	mvwaddch(mainmenu, max_y - 4, 47, '7');
	mvwaddch(mainmenu, max_y - 3, 47, '8');
	mvwaddch(mainmenu, max_y - 2, 47, '9');
	wattroff(mainmenu, A_BOLD);
	wattroff(mainmenu, COLOR_PAIR(STYLE_NORMAL));
    } else if (win > -1) {
      cnt = 0;
      while (getaclentry("stat_section", &entry) && ARG0) 
      	if (win == cnt++) break;
  	mvwprintw(mainmenu, 3, 33, "Xfer Statistics (%s)", ARG0);
    	mvwprintw(mainmenu, 6, 15, "Up:   %6d F     %10.1f MB     %8.0f K/s", ug.statsection[win].files_up_day, (float)(ug.statsection[win].bytes_up_day / 1024.0), (ug.statsection[win].seconds_up_day) ? (float)(ug.statsection[win].bytes_up_day / ug.statsection[win].seconds_up_day) : 0.0); 
	mvwprintw(mainmenu, 7, 15, "Dn:   %6d F     %10.1f MB     %8.0f K/s", ug.statsection[win].files_down_day, (float)(ug.statsection[win].bytes_down_day / 1024.0), (ug.statsection[win].seconds_down_day) ? (float)(ug.statsection[win].bytes_down_day / ug.statsection[win].seconds_down_day) : 0.0);
 	mvwprintw(mainmenu, 9, 15, "Up:   %6d F     %10.1f MB     %8.0f K/s", ug.statsection[win].files_up_wk, (float)(ug.statsection[win].bytes_up_wk / 1024.0), (ug.statsection[win].seconds_up_wk) ? (float)(ug.statsection[win].bytes_up_wk / ug.statsection[win].seconds_up_wk) : 0.0);
	mvwprintw(mainmenu,10, 15, "Dn:   %6d F     %10.1f MB     %8.0f K/s", ug.statsection[win].files_down_wk, (float)(ug.statsection[win].bytes_down_wk / 1024.0), (ug.statsection[win].seconds_down_wk) ? (float)(ug.statsection[win].bytes_down_wk/ ug.statsection[win].seconds_down_wk) : 0.0);     
	mvwprintw(mainmenu,12, 15, "Up:   %6d F     %10.1f MB     %8.0f K/s", ug.statsection[win].files_up_month, (float)(ug.statsection[win].bytes_up_month / 1024.0), (ug.statsection[win].seconds_up_month) ? (float)(ug.statsection[win].bytes_up_month / ug.statsection[win].seconds_up_month) : 0.0);
	mvwprintw(mainmenu,13, 15, "Dn:   %6d F     %10.1f MB     %8.0f K/s", ug.statsection[win].files_down_month, (float)(ug.statsection[win].bytes_down_month / 1024.0), (ug.statsection[win].seconds_down_month) ? (float)(ug.statsection[win].bytes_down_month / ug.statsection[win].seconds_down_month) : 0.0);
	mvwprintw(mainmenu,15, 15, "Up:   %6d F     %10.1f MB     %8.0f K/s", ug.statsection[win].files_up, (float)(ug.statsection[win].bytes_up / 1024.0), (ug.statsection[win].seconds_up) ? (float)(ug.statsection[win].bytes_up / ug.statsection[win].seconds_up) : 0.0);
	mvwprintw(mainmenu,16, 15, "Dn:   %6d F     %10.1f MB     %8.0f K/s", ug.statsection[win].files_down, (float)(ug.statsection[win].bytes_down / 1024.0), (ug.statsection[win].seconds_down) ? (float)(ug.statsection[win].bytes_down / ug.statsection[win].seconds_down) : 0.0);

	mvwprintw(mainmenu,19, 15, "Ratio: %d", ug.statsection[win].ratio);
	mvwprintw(mainmenu,20, 15, "Times Nuked: %d", ug.statsection[win].times_nuked);
	commafmt(numbuff, 64, ug.statsection[win].bytes_nuked);
	mvwprintw(mainmenu,21, 15, "Bytes Nuked: %-s", numbuff);
	mvwprintw(mainmenu,22, 15, "Last Nuked:  %-s", trim(ctime(&ug.statsection[win].last_nuked)));

	if (!strcasecmp(ARG2, "yes")) {
		commafmt(numbuff, 64, ug.statsection[win].credits);
 		mvwprintw(mainmenu,22, 15, "Credits:  %-s KB", numbuff);
    	} else {
		commafmt(numbuff, 64, ug.statsection[0].credits);
		mvwprintw(mainmenu,22, 15, "Credits:  %-s KB", numbuff);
	} 
	wattron(mainmenu, COLOR_PAIR(STYLE_NORMAL));
	mvwprintw(mainmenu, 5, 14, "---------------------------[ Today ]-----------------------------");
	mvwprintw(mainmenu, 8, 14, "---------------------------[ Week ]------------------------------");
	mvwprintw(mainmenu, 11,14, "---------------------------[ Month ]-----------------------------");
	mvwprintw(mainmenu, 14,14, "---------------------------[ Alltime ]---------------------------");
    	wattron(mainmenu, A_BOLD);
 	mvwaddch(mainmenu, 5, 43, 'T');
	mvwaddch(mainmenu, 8, 43, 'W');
	mvwaddch(mainmenu,11, 43, 'M');
	mvwaddch(mainmenu,14, 43, 'A'); 
	mvwaddch(mainmenu, 19, 15, 'R');    
	mvwaddch(mainmenu, 20, 21, 'N');
	mvwaddch(mainmenu, 21, 15, 'B');
	mvwaddch(mainmenu, 22, 15, 'C');

	wattroff(mainmenu, A_BOLD);
	wattroff(mainmenu, COLOR_PAIR(STYLE_NORMAL));
    }
wnoutrefresh(mainmenu);
doupdate();
}

/*-----------------------------------------------------------------------*/
void clear_window(void) {
	int x;
 	for (x=3; x < (max_y - 1); x++)
		mvwprintw(mainmenu, x, 14, "                                                                 ");
}
