#include "userfile.h"
#include "extensions.h"
#include "userdata.h"
#include <dirent.h>
#include <stdlib.h>
#include <unistd.h>
#include <string.h>
#include <stdio.h>
#include <sys/stat.h>
#include <sys/param.h>

struct USERDATA *first = NULL;
struct USERDATA *last = NULL;

char config_file[255];
char rootpath[MAXPATHLEN];
char datapath[MAXPATHLEN];
void sysconfig(void);
void display_user(void);
void free_user_data(void);
void enum_user_struct(void);
void userstore (struct USERDATA *, struct USERDATA **, struct USERDATA **);

extern void init_scr(void);
extern char version[];
/**************************************************************************
	MAIN()   USEREDIT FUNCTION
***************************************************************************/
int main ( int argc, char **argv)
{
	if (argc > 1) {
		if (strncmp("-r", argv[1], 2)) {
			printf("%s\n\n", version);
			printf("Command line options:  -r {alternate config file}\n");
			printf("Help:   ESC or 'Q' to quit.\n");
			printf("	ARROW, PGUP/PGDN, HOME/END Keys navigate user scroll.\n");
			printf("        ARROW KEYS and TAB key scrolls through stat windows.\n");
			exit(0);
		} 
		else {
			if (argv[1][2]) 
				strncpy(config_file, argv[1]+2, sizeof(config_file));
			else
				strncpy(config_file, argv[2], sizeof(config_file));
			config_file[sizeof(config_file)-1] = '\0';
		}
	} else 	strcpy(config_file, "/etc/glftpd.conf");
	sysconfig();
	fill_user_struct();
	enum_user_struct();
	mainMenu();
	free_user_data();
	exit(0);
}
/*-----------------------------------------------------------------------*/
int fill_user_struct(void)
{
	DIR *dirf;
	struct dirent *dn;
	struct stat st;
	char path[MAXPATHLEN];
	struct USERDATA *users;

	sprintf(path, "%s%s/users", rootpath, datapath);
	dirf = opendir(path);
	if (!dirf) {
		fprintf(stderr, "Error opening User Dir.");
		return 1;
	}
	while (1)
	{
		dn = readdir(dirf);
		if (!dn)
			break;
		sprintf(path, "%s%s/users/%s", rootpath, datapath, dn->d_name);
		stat(path, &st);
		if (!S_ISREG(st.st_mode))
			continue;
		users = (struct USERDATA *) malloc(sizeof(struct USERDATA));
		if (!users)
			break;
		if (strncasecmp("default.", dn->d_name, 8) == 0)
			continue;
	 	strncpy(users->username, dn->d_name, sizeof(users->username));
		(void)userstore(users, &first, &last);
	}
	closedir(dirf);
	return 0;
}
/*-----------------------------------------------------------------------*/
void free_user_data(void)
{
	struct USERDATA *users, *users_next;

	if (first == NULL)
		return;
	users = first;
	while (users) {
		users_next = users->next;
		free(users);
		users = users_next;
	}
}
/*-----------------------------------------------------------------------*/
void sysconfig( void )
{
        struct aclmember *entry = NULL;

        readacl(config_file);
        getaclentry("rootpath", &entry);
        strncpy(rootpath, ARG0, sizeof(rootpath));
        entry = NULL;
        getaclentry("datapath", &entry);
        strncpy(datapath, ARG0, sizeof(datapath));
}

/*-------------------------------------------------------------------------*/
void enum_user_struct(void) {
	struct USERDATA *users, *ptr;
	int num = 0;

	users = first;
	while (users) {
		ptr = users->next;
		users->num = num++;
		users = ptr;
	}
}
/****************************************************************************
  USERSTORE
  Adds a user to the list (in sorted order)
****************************************************************************/

void userstore (struct USERDATA *i, struct USERDATA **start, struct USERDATA **finish) {
  struct USERDATA *old = NULL, *p;


  if (*finish == NULL)
    {
      /*
       * First element in list?
       */
      i->next = NULL;
      i->prev = NULL;
      *finish = i;
      *start = i;
      return;
    }
  /*
   * Start at top of list
   */
  p = *start;

  while (p)
    {
      if (strcmp (p->username, i->username) < 0)
        {
          old = p;
          p = p->next;
        }
      else
        {
          if (p->prev)
            {
              p->prev->next = i;
              i->next = p;
              i->prev = p->prev;
              p->prev = i;
              return;
            }
          i->next = p;
          i->prev = NULL;
          p->prev = i;
          *start = i;
          return;
        }
    }

  old->next = i;
  i->next = NULL;
  i->prev = old;
  *finish = i;
}
/*-- end of userstore() ---------------------------------------------------*/
void
parse(int num)
{
        struct USERDATA *users = first;

        while (users) 
	{
		if (users->num == num) {
        	        load_userfile(users->username);
			break;
		}
		users = users->next;
	}
}

